@@ -1,5 +1,7 @@
 #' Compute Indicators for Qualified Item Missingness
 #'
+#' [Indicator]
+#'
 #' @param resp_vars [variable list] the name of the measurement variables
 #' @param study_data [data.frame] the data frame that contains the measurements
 #' @param meta_data [data.frame] the data frame that contains metadata
@@ -39,9 +41,9 @@
 #' resp_vars <- sample(colnames(ship), ceiling(ncol(ship) / 20), FALSE)
 #' mistab <- prep_get_data_frame("missing_matchtable1")
 #' valid_replacement_codes <-
-#'   mistab[mistab$CODE_INTERPRET != "P", "CODE_VALUE",
+#'   mistab[mistab$CODE_INTERPRET != "I", "CODE_VALUE",
 #'     drop =
-#'     TRUE] # sample only replacement codes on item level. P uses the actual
+#'     TRUE] # sample only replacement codes on item level. I uses the actual
 #'           # values
 #' for (rv in resp_vars) {
 #'   values <- sample(as.numeric(valid_replacement_codes), number_of_mis,
@@ -70,10 +72,6 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
   # TODO: Allow also here dates values as missing codes
   # TODO (for the example): Add also missing table assignments to the synthetic cases with PART_VAR working following the old implementation
 
-  # TODO (JM): See library(roxygen2)
-  # ?"tags-index-crossref"
-  #
-
   # Preparation of the input ----
 
   prep_prepare_dataframes(.replace_missings = FALSE)
@@ -90,7 +88,7 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
                             allow_any_obs_na = TRUE)
 
   # allowed AAPOR-States
-  AAPOR_STATES <- c("P", "PP", "PL", "R", "BO", "NC", "O", "UH", "OH", "NE")
+  AAPOR_STATES <- c("I", "P", "PL", "R", "BO", "NC", "O", "UH", "UO", "NE")
 
   # Loop over all resp_vars ----
 
@@ -103,30 +101,36 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
         .cur_miss <- meta_data[meta_data[[label_col]] == rv, # find the missing code table assigned to the current study variable
                               MISSING_LIST_TABLE]
         if (length(.cur_miss) == 0) {
-          .cur_miss <- NA_character_
+          .cur_miss <- NA_character_ # nocov: Cannot be reached, because if rv is not in the metadata, we fail already in util_correct_variable_use, otherwise, we may have NA in MISSING_LIST_TABLE, but not length(.) == 0
         }
         cur_miss <- try(util_expect_data_frame(x = .cur_miss, # this is up to here the name of the missing table, not the table itself
                                                dont_assign = TRUE, # don't replace the value of .cur_miss by the data frame, just return it
                                                col_names = # check/ensure, content of the data frame matches expectations
                                                list(
-          CODE_VALUE = is.numeric,
+          CODE_VALUE = util_is_valid_missing_codes,
           CODE_INTERPRET = function(x) {
             is.character(x) & x %in% AAPOR_STATES
           }
         ),
+        custom_errors = list(
+          CODE_VALUE = "CODE_VALUE must be numeric or DATETIME",
+          CODE_INTERPRET = paste("CODE_INTERPRET must be one of",
+                                 util_pretty_vector_string(AAPOR_STATES))
+        ),
         convert_if_possible = list( # try to convert values to match expectations, if produces NAs, an error is thrown. Thereafter, checks from col_names above will re-run.
-          CODE_VALUE = as.numeric,
+          CODE_VALUE = util_as_valid_missing_codes,
           CODE_INTERPRET = function(x) {
             trimws(toupper(x))
           }
         )), silent = TRUE)
         if (inherits(cur_miss, "try-error")) { # some problems with fetching and checking/fixing the data frame by util_expect_data_frame
+          # nocov start: Cannot be reached, because most potential problems will have occurred already in prep_prepare_data_frames > prep_meta_data_v1_to_item_level_meta_data
           if (!is.na(.cur_miss)) util_warning(
   "Could not load missing-match-table %s for response variable %s: %s",
                dQuote(.cur_miss),
                dQuote(rv),
                conditionMessage(attr(cur_miss, "condition")),
-               applicability_problem = TRUE) else
+               applicability_problem = TRUE) else # nocov end
             util_warning(
               "No missing-match-table for response variable %s",
               dQuote(rv),
@@ -138,22 +142,23 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
             util_filter_missing_list_table_for_rv(cur_miss, rv, rv_vn)
 
           # really compute stuff ----
-          code_for_P <- head(cur_miss[cur_miss[["CODE_INTERPRET"]] == "P",
-                                 "CODE_VALUE", TRUE], 1) # get the first code for P to replace the real measurements by this code
+          code_for_P <- head(cur_miss[cur_miss[["CODE_INTERPRET"]] == "I",
+                                 "CODE_VALUE", TRUE], 1) # get the first code for I to replace the real measurements by this code
           if (length(code_for_P) == 0) {
-            code_for_P <- max(cur_miss$CODE_VALUE, na.rm = TRUE) + 1
+            code_for_P <- util_find_free_missing_code(cur_miss$CODE_VALUE)
              P_df <- data.frame(
                CODE_VALUE = code_for_P,
                CODE_LABEL = "Participation",
-               CODE_INTERPRET = "P",
+               CODE_INTERPRET = "I",
                stringsAsFactors = FALSE
              )
              cur_miss <- util_rbind(cur_miss,
                                P_df)
-             code_for_P <- head(cur_miss[cur_miss[["CODE_INTERPRET"]] == "P",
-                                         "CODE_VALUE", TRUE], 1) # get the first code for P to replace the real measurements by this code
+             code_for_P <- head(cur_miss[cur_miss[["CODE_INTERPRET"]] == "I",
+                                         "CODE_VALUE", TRUE], 1) # get the first code for I to replace the real measurements by this code
           }
           if (length(code_for_P) != 1) {
+            # nocov start: Cannot be reached, because if there would've been no I code, we would have one after the last block. If we had > 1, we would have chosen the first one already, wlog
             util_error(
               applicability_problem = TRUE,
               c("Could not find any participation related code in %s %s",
@@ -162,11 +167,12 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
               dQuote(.cur_miss),
               dQuote(rv)
               )
+            # nocov end
           }
 
           rx <- ds1[[rv]]
 
-          rx[!is.na(rx) & !(rx %in% cur_miss[cur_miss[["CODE_INTERPRET"]] != "P",
+          rx[!is.na(rx) & !(rx %in% cur_miss[cur_miss[["CODE_INTERPRET"]] != "I",
                                              "CODE_VALUE", TRUE])] <- code_for_P
 
           rx <- suppressWarnings(as.numeric(rx)) # TODO: Handle dates, handle errors
@@ -178,13 +184,40 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
 
           r <- as.list(setNames(r$Freq, nm = r$Var1)) # convert the data frame with columns Var1 and Freq to a named list, names are the AAPOR codes, values are the frequencies
 
+          r$N <- sum(!util_empty(ds1[[rv]]))
+          r$N2 <- util_count_expected_observations(rv,
+                                                   study_data = ds1,
+                                                   meta_data = meta_data,
+                                                   label_col = label_col,
+                                                   expected_observations =
+                                                     expected_observations)
+
+          if (length(r$P) == 1 && !is.na(r$P) && r$P != 0) {
+            util_warning(
+              c("%0.2f%% of the values in %s have a missing code for",
+                "%s assigned, which is impossible on item level. Treating",
+                "these cases as %s"),
+              round(r$P / r$N * 100, 2),
+              sQuote(rv),
+              sQuote("P"),
+              sQuote("UO"),
+              applicability_problem = TRUE)
+            if (length(r$UO) == 1 && !is.na(r$UO) && r$UO != 0) {
+              r$UO <- r$UO + r$P
+            } else {
+              r$UO <- r$P
+            }
+            r$P <- 0
+          }
+
+
           # Generate replacement list
           rclean <- r
 
           # Enter 0 for NULL in replacement list to always enable computations
           for (name in AAPOR_STATES) {
             if (is.null(rclean[[name]])) {
-              rclean[[name]]<-0
+              rclean[[name]] <- 0
             }
           }
 
@@ -192,13 +225,15 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
 
           # DQI-2002    Based on participation only
           # NRR1 for all who did not participate
-          # Minimal information: Study, Segment: (P | PP ) -> NOTE NE not used in denominator
-          # RR1<-(P+PP)/((P+PP+PL) + (R+BO+NC+O) + (UH+UO)
-          if(!is.null(r$P) | !is.null(r$PP) ) {
-            r$RR1 <- (rclean$P + rclean$PP) / ((rclean$P+rclean$PP+rclean$PL) + (rclean$R+rclean$BO+rclean$NC+rclean$O) + (rclean$UH+rclean$OH))
+          # Minimal information: Study, Segment: (I | P ) -> NOTE NE not used in denominator
+          # RR1<-(I+P)/((I+P+PL) + (R+BO+NC+O) + (UH+UO)
+          if(!is.null(r$I) | !is.null(r$P) ) {
+            r$RR1 <- (rclean$I + rclean$P) / ((rclean$I+rclean$P+rclean$PL) + (rclean$R+rclean$BO+rclean$NC+rclean$O) + (rclean$UH+rclean$UO))
             r$NRR1 <- 1 - r$RR1
             r$PCT_com_qum_nonresp <- 100 * r$NRR1
           } else {
+            # nocov start
+            # we will always have at least I == 0, because this is ensured above.
             util_warning(
 "Preconditions to generate Nonresponse Rate 1 not given for study variable %s",
               dQuote(rv),
@@ -206,60 +241,59 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
             r$NRR1 <- NA_real_
             r$RR1 <- NA_real_
             r$PCT_com_qum_nonresp <- NA_real_
+            # nocov end
           }
 
           # NRR2 based on participation and scheduled dates
-          # Minimal information: Study, Segment: (P | PP | PL ) -> NOTE NE not used in denominator
-          # RR2<-(P+PP+PL)/((P+PP+PL) + (R+BO+NC+O) + (UH+UO)
-          if(!is.null(r$P) | !is.null(r$PP)  | !is.null(r$PL) ) {
-            r$RR2 <- (rclean$P + rclean$PP + rclean$PL) / ((rclean$P+rclean$PP+rclean$PL) + (rclean$R+rclean$BO+rclean$NC+rclean$O) + (rclean$UH+rclean$OH))
+          # Minimal information: Study, Segment: (I | P | PL ) -> NOTE NE not used in denominator
+          # RR2<-(I+P+PL)/((I+P+PL) + (R+BO+NC+O) + (UH+UO)
+          if(!is.null(r$I) | !is.null(r$P)  | !is.null(r$PL) ) {
+            r$RR2 <- (rclean$I + rclean$P + rclean$PL) / ((rclean$I+rclean$P+rclean$PL) + (rclean$R+rclean$BO+rclean$NC+rclean$O) + (rclean$UH+rclean$UO))
             r$NRR2 <- 1 - r$RR2
           } else {
+            # nocov start
+            # we will always have at least I == 0, because this is ensured above.
             util_warning(
       "Preconditions to generate Nonresponse Rate 1 not given for variable %s",
               dQuote(rv),
               applicability_problem = TRUE)
             r$NRR2 <- NA_real_
             r$RR2 <- NA_real_
+            # nocov end
           }
 
 
           # DQI-2003    Refusal rate
           # NRR1 for all who refused at any stage
           # Minimal information: Study, Segment: R | BO
-          # REF1<-(R+BO)/((P+PP+PL) + (R+BO+NC+O) + (UH+UO)
+          # REF1<-(R+BO)/((I+P+PL) + (R+BO+NC+O) + (UH+UO)
           if(!is.null(r$R) | !is.null(r$BO) ) {
-            r$REF1 <- (rclean$R + rclean$BO) / ((rclean$P+rclean$PP+rclean$PL) + (rclean$R+rclean$BO+rclean$NC+rclean$O) + (rclean$UH+rclean$OH))
+            r$REF1 <- (rclean$R + rclean$BO) / ((rclean$I+rclean$P+rclean$PL) + (rclean$R+rclean$BO+rclean$NC+rclean$O) + (rclean$UH+rclean$UO))
             r$PCT_com_qum_refusal <- 100 * r$REF1
           } else {
             util_warning(
-      "Preconditions to generate Nonresponse Rate 1 not given for variable %s",
+      "Preconditions to generate Nonresponse Rate 1 not given for variable %s", # TODO: Write nicer warnings, here.
               dQuote(rv),
               applicability_problem = TRUE)
             r$REF1 <- NA_real_
             r$PCT_com_qum_refusal <- NA_real_
           }
 
-          r$N <- sum(!util_empty(ds1[[rv]]))
-          r$N2 <- util_count_expected_observations(rv,
-                                                  study_data = ds1,
-                                                  meta_data = meta_data,
-                                                  label_col = label_col,
-                                                  expected_observations =
-                                                    expected_observations)
-
           r <- cbind( # create a new data frame with a column Variables and all the rates and values for the current rv 1 row, only.
             data.frame(Variables = rv, stringsAsFactors = FALSE, check.names = FALSE, row.names = NULL),
             data.frame(r, stringsAsFactors = FALSE, check.names = FALSE, row.names = NULL)
           )
         }
       } else { # study data lacks the status variable for the  current variable, omit this segment
+        # nocov start
+        # unreachabel, becasue if rv would not be in colnames(ds1), util_correct_variable_use would have stopped, already
         util_warning("Missing variable %s from %s",
                      dQuote(rv),
                      sQuote("study_data"),
                      applicability_problem = TRUE,
                      intrinsic_applicability_problem = TRUE)
         return(NULL)
+        # nocov end
       }
     })
 
@@ -301,7 +335,7 @@ com_qualified_item_missingness <-    function(resp_vars, study_data,
             paste0(round(cl, 2), "%")
            })
 
-  SummaryTable$GRADING <- SummaryTable[["PCT_com_qum_nonresp"]] > 10 # TODO: only to avoid empty output in issue matrix
+  # SummaryTable$GRADING <- SummaryTable[["PCT_com_qum_nonresp"]] > 10 # only to avoid empty output in issue matrix, can be removed, now.
 
   return(list( # return the results
     SummaryTable = SummaryTable,
