@@ -7,5 +7,9 @@
 #' For a quick start please read [dq_report2] and maybe the vignettes
 #' or the package's [website](https://dataquality.qihs.uni-greifswald.de/).
 #'
+#' @keywords internal
+"_PACKAGE"
+
+## usethis namespace: start
+## usethis namespace: end
 NULL
-#> NULL
