@@ -13,6 +13,7 @@ prep_link_escape <- function(s, html = FALSE) {
   r <- gsub("\u00B0", "", r, fixed = TRUE)
   r <- gsub("%", "", r, fixed = TRUE)
   r <- gsub("/", "", r, fixed = TRUE)
+  r <- gsub("\\", "", r, fixed = TRUE)
   r <- gsub("'", "", r, fixed = TRUE)
   r <- gsub(":", "", r, fixed = TRUE)
   r <- gsub("\u20ac", "", r, fixed = TRUE)
@@ -29,7 +30,7 @@ prep_link_escape <- function(s, html = FALSE) {
   r <- gsub("\u00b5", "", r, fixed = TRUE)
   r <- gsub(";", "", r, fixed = TRUE)
   r <- gsub("\u00b0", "", r, fixed = TRUE)
-  r <- gsub("?", "", r, fixed = TRUE)
+  r <- gsub("?", "Q", r, fixed = TRUE)
   r <- gsub("#", "", r, fixed = TRUE)
   # r <- gsub("[\"/ <>&;°@µ~]", "", r, perl = TRUE)
   gsub("\\s+", "", r, perl = TRUE)
