@@ -14,8 +14,15 @@
 #'             `htmltools::renderDependencies`
 #' @param progress_msg [closure] to call with progress information
 #' @param progress [closure] to call with progress information
+#' @param title [character] the web browser's window name
+#' @param by_report [logical] this report html is part of a set of reports,
+#'                            add a back-link
 #'
 #' @return `invisible(file_name)`
+#'
+#' @family reporting_functions
+#' @concept process
+#' @keywords internal
 util_create_page_file <- function(page_nr,
                                   pages,
                                   rendered_pages,
@@ -27,7 +34,9 @@ util_create_page_file <- function(page_nr,
                                   packageName,
                                   deps,
                                   progress_msg,
-                                  progress) {
+                                  progress,
+                                  title,
+                                  by_report) {
 
   page <- names(pages)[page_nr] # the name of the page-file being created
 
@@ -41,14 +50,54 @@ util_create_page_file <- function(page_nr,
 
   pg[["dependencies"]] <- NULL
 
+  if (by_report) {
+    backlink <- htmltools::div(
+      style = htmltools::css(
+        position = "fixed",
+        right = "0",
+        bottom = "0"
+      ),
+      htmltools::a(
+        href = "#",
+        onclick =
+          'window.location.href = "../../index.html"',
+        "Back to reports' overview"
+      )
+    )
+  } else {
+    backlink <- NULL
+  }
+
+  if (!is.null(attr(report, "title")) &&
+      !isTRUE(attr(attr(report, "title"), "default"))) {
+    header_text <- attr(report, "title")
+    if (!is.null(attr(report, "subtitle")) &&
+        !isTRUE(attr(attr(report, "subtitle"), "default"))) {
+      header_text <- paste0(header_text, ": ", attr(report, "subtitle"))
+    }
+  } else {
+    header_text <- NULL
+  }
+
+  if (!is.null(header_text)) {
+    header <- htmltools::tagList(
+      htmltools::p(class = "dq-title",
+                   htmltools::tags$a(href = "report.html", header_text))
+    )
+  } else {
+    header <- NULL
+  }
+
   html_report <- htmltools::htmlTemplate(template_file,
                                   document_ = TRUE,
                                   spage = pg,
                                   logo = logo,
                                   menu = .menu_env$menu(pages),
                                   loading = loading,
-                                  deps = deps
-                                  )
+                                  deps = deps,
+                                  title = title,
+                                  backlink = backlink,
+                                  header = header)
 
   #fix: sort reportsummarytable by first (sysmiss) and varaible column
 
