@@ -1,7 +1,10 @@
+#' @family reporting_functions
+#' @concept process
+#' @keywords internal
 util_remove_dataquieR_result_class <- function(x) { # TODO: The [[ for dataquieR_result should not work this way, a wrapper class would be needed, instead. Then, this funcion can be abandondd
   remove_class <- function(x) {
     if (!is.null(x)) {
-      if (is.list(x) && !inherits(x, "patchwork")) {
+      if (is.list(x) && !inherits(x, "patchwork") && !inherits(x, "ggmatrix")) {
         x[] <- lapply(x, remove_class)
       }
       newclass <- setdiff(class(x), "dataquieR_result")
