@@ -4,6 +4,14 @@ test_that("con_limit_deviations works", {
   withr::local_timezone("CET")
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
   expect_message(
     MyValueLimits <- con_limit_deviations(resp_vars  = c("AGE_0", "SBP_0",
                                                          "DBP_0", "SEX_0",
@@ -12,30 +20,19 @@ test_that("con_limit_deviations works", {
                                                          "SMOKE_SHOP_0"),
                                           label_col  = "LABEL",
                                           study_data = study_data,
-                                          meta_data  = meta_data,
-                                          limits     = "HARD_LIMITS"),
-    regexp = sprintf("(%s|%s|%s|%s)",
-                     paste("The variables SEX_0 have no defined limits."),
-                     paste("N = 9 values in QUEST_DT_0 lie outside hard",
-                           "limits and were removed."),
-                     paste("N = 3 values in EDUCATION_1 lie outside hard",
-                           "limits and were removed."),
-                     paste("N = 24 values in SMOKE_SHOP_0 lie outside hard",
-                           "limits and were removed.")
-                     ),
-    all = TRUE,
-    perl = TRUE
-  )
+                                          meta_data  = meta_data),
+    regexp = "No limits specified for SEX_0")
 
-  expect_equal(sum(MyValueLimits$SummaryData$GRADING), 3)
-  expect_true(all(MyValueLimits$FlaggedStudyData$QUEST_DT_0[which(as.logical(
-    MyValueLimits$FlaggedStudyData$QUEST_DT_0_below_HARD))] <
-      as.POSIXct("2018-01-01")))
-  expect_true(all(MyValueLimits$FlaggedStudyData$QUEST_DT_0[which(!as.logical(
-    MyValueLimits$FlaggedStudyData$QUEST_DT_0_below_HARD))] >=
-      as.POSIXct("2018-01-01")))
-  expect_equal(rowSums(MyValueLimits$SummaryTable[, c("NUM_con_rvv_inum", "NUM_con_rvv_itdat")], na.rm = TRUE),
-               MyValueLimits$SummaryData$`Below hard limits (N)` + MyValueLimits$SummaryData$`Above hard limits (N)`)
+   expect_equal(
+     unname(rowSums(MyValueLimits$SummaryTable[
+       order(MyValueLimits$SummaryTable$Variables),
+       c("NUM_con_rvv_inum", "NUM_con_rvv_itdat")], na.rm = TRUE)),
+     MyValueLimits$SummaryData %>%
+       dplyr::filter(Section != "within" & Limits == "HARD_LIMITS") %>%
+       dplyr::group_by(Variables) %>%
+       dplyr::summarise(n = sum(Number)) %>%
+       dplyr::pull(n)
+   )
 })
 
 test_that("con_limit_deviations and timevars with < 20 integer sec-values ok", {
@@ -52,6 +49,14 @@ test_that("con_limit_deviations and timevars with < 20 integer sec-values ok", {
   }
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   vn <- subset(meta_data, LABEL == "QUEST_DT_0", VAR_NAMES, TRUE)
 
@@ -72,8 +77,8 @@ test_that("con_limit_deviations and timevars with < 20 integer sec-values ok", {
 
   skip_on_cran()
   skip_if_not_installed("vdiffr")
-  skip_if_not(capabilities()["long.double"])
-  skip_if_not(capabilities()["long.double"])
+  # TODO: skip_if_not(capabilities()["long.double"])
+  # TODO: skip_if_not(capabilities()["long.double"])
   suppressWarnings(
     vdiffr::expect_doppelganger(
       "con_limit_deviations QUEST_DT_0",
@@ -88,6 +93,14 @@ test_that("con_limit_deviations works w/o resp_vars", {
   withr::local_timezone("CET")
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   resp_vars <- c("AGE_0", "SBP_0",
                  "DBP_0", "SEX_0",
@@ -109,16 +122,11 @@ test_that("con_limit_deviations works w/o resp_vars", {
     MyValueLimits <- con_limit_deviations(label_col  = "LABEL",
                                           study_data = sd0,
                                           meta_data  = md0,
-                                          limits     = "HARD_LIMITS"),
-    regexp = sprintf("(%s|%s|%s|%s|%s)",
-                     paste("All variables with HARD_LIMITS in",
-                           "the metadata are used."),
-                     paste("N = 9 values in QUEST_DT_0 lie outside hard",
-                           "limits and were removed."),
-                     paste("N = 3 values in EDUCATION_1 lie outside hard",
-                           "limits and were removed."),
-                     paste("N = 24 values in SMOKE_SHOP_0 lie outside hard",
-                           "limits and were removed."),
+                                          limits     = "HARD_LIMITS",
+                                          return_flagged_study_data = TRUE),
+    regexp = sprintf("(%s|%s)",
+                     paste("All variables for which limits are specified",
+                           "in the metadata are used."),
                      paste("Not all entries in.+KEY_STUDY_SEGMENT.+are",
                            "found in.+VAR_NAMES.+")
     ),
@@ -126,8 +134,10 @@ test_that("con_limit_deviations works w/o resp_vars", {
     perl = TRUE
   )
 
-  expect_equal(sum(MyValueLimits$SummaryData$GRADING),
-               3)
+  expect_equal(sum(rowSums(
+    MyValueLimits$SummaryTable[,
+      grepl("^FLG\\_.*$", colnames(MyValueLimits$SummaryTable))],
+    na.rm = TRUE) > 0), 4)
   expect_true(all(MyValueLimits$FlaggedStudyData$QUEST_DT_0[which(as.logical(
     MyValueLimits$FlaggedStudyData$QUEST_DT_0_below_HARD))] <
       as.POSIXct("2018-01-01")))
@@ -143,6 +153,14 @@ test_that("con_limit_deviations handles errors", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   resp_vars <- c("AGE_0", "SBP_0",
                  "DBP_0", "SEX_0",
@@ -158,16 +176,18 @@ test_that("con_limit_deviations handles errors", {
   sd0 <- study_data[, resp_vars_names, FALSE]
   md0 <- meta_data[meta_data$VAR_NAMES %in% resp_vars_names, , FALSE]
   md0$JUMP_LIST <- SPLIT_CHAR # signals unambiguously no codes intentionally
-  md0$HARD_LIMITS <- NA # no limits
+  # no limits:
+  md0$HARD_LIMITS <- NA
+  md0$DETECTION_LIMITS <- NA
+  md0$SOFT_LIMITS <- NA
 
   expect_message(expect_error(
     MyValueLimits <- con_limit_deviations(resp_vars = resp_vars,
                                           label_col  = "LABEL",
                                           study_data = sd0,
-                                          meta_data  = md0,
-                                          limits     = "HARD_LIMITS"),
-    regexp = "No variables with defined HARD_LIMITS.",
-    perl = TRUE
+                                          meta_data  = md0),
+    regexp = paste("The limit deviation check cannot be performed",
+                   "without a metadata column specifying suitable limits")
   ),
   regexp = "Not all entries in .+KEY_STUDY_SEGMENT.+0.+7.*")
 
@@ -188,8 +208,7 @@ test_that("con_limit_deviations handles errors", {
         con_limit_deviations(resp_vars = md0$LABEL, # missing, only resp_vars with suitable data type are picked by the function now.
                              label_col  = "LABEL",
                              study_data = sd0,
-                             meta_data  = md0,
-                             limits     = "HARD_LIMITS")),
+                             meta_data  = md0)),
       regexp = paste("Variable .+PSEUDO_ID.+ .+string.+ does not have an",
                            "allowed type"),
     perl = TRUE
@@ -214,6 +233,14 @@ test_that("con_limit_deviations and values < 0", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   sd0 <- study_data
   x <- sd0[[prep_map_labels("EDUCATION_0", meta_data, VAR_NAMES, LABEL)]]
@@ -236,7 +263,7 @@ test_that("con_limit_deviations and values < 0", {
 
   skip_on_cran()
   skip_if_not_installed("vdiffr")
-  skip_if_not(capabilities()["long.double"])
+  # TODO: skip_if_not(capabilities()["long.double"])
   suppressWarnings(
     vdiffr::expect_doppelganger(
       "con_limit_deviations for reverse order ok",
@@ -252,6 +279,14 @@ test_that("con_limit_deviations and values < 0 with max -1", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   sd0 <- study_data
   x <- sd0[[prep_map_labels("EDUCATION_0", meta_data, VAR_NAMES, LABEL)]]
@@ -274,7 +309,7 @@ test_that("con_limit_deviations and values < 0 with max -1", {
 
   skip_on_cran()
   skip_if_not_installed("vdiffr")
-  skip_if_not(capabilities()["long.double"])
+  # TODO: skip_if_not(capabilities()["long.double"])
   suppressWarnings(
     vdiffr::expect_doppelganger(
       "con_limit_deviations rev order w/ -1 ==> 0 max ok",
@@ -290,6 +325,14 @@ test_that("con_limit_deviations with no lower limit", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   sd0 <- study_data
   x <- sd0[[prep_map_labels("EDUCATION_0", meta_data, VAR_NAMES, LABEL)]]
@@ -299,15 +342,12 @@ test_that("con_limit_deviations with no lower limit", {
   sd0[[prep_map_labels("EDUCATION_0", meta_data, VAR_NAMES, LABEL)]] <- x
   md0 <- meta_data
 
-  expect_message(
+  expect_silent(
     MyValueLimits1 <- con_limit_deviations(resp_vars  = "EDUCATION_0",
                                           label_col  = "LABEL",
                                           study_data = sd0,
                                           meta_data  = md0,
-                                          limits     = "HARD_LIMITS"),
-    regexp = paste("N = 4 values in EDUCATION_0 lie outside hard limits",
-                   "and were removed."),
-    perl = TRUE
+                                          limits     = "HARD_LIMITS")
   )
 
   md0[md0$LABEL == "EDUCATION_0", HARD_LIMITS] <- "[;6]"
@@ -318,11 +358,12 @@ test_that("con_limit_deviations with no lower limit", {
                                          meta_data  = md0,
                                          limits     = "HARD_LIMITS")
 
-  expect_equal(MyValueLimits1$SummaryData$`Above hard limits (N)`,
-               MyValueLimits2$SummaryData$`Above hard limits (N)`)
+  expect_equal(MyValueLimits1$SummaryTable$NUM_con_rvv_inum, 4)
+  expect_equal(MyValueLimits2$SummaryTable$NUM_con_rvv_inum, 0)
 
-  expect_equal(MyValueLimits1$SummaryData$`Below hard limits (N)`, 4)
-  expect_equal(MyValueLimits2$SummaryData$`Below hard limits (N)`, 0)
+  expect_equal(MyValueLimits1$SummaryData$Number[
+    which(MyValueLimits1$SummaryData$Section == "above")], MyValueLimits2$SummaryData$Number[
+      which(MyValueLimits2$SummaryData$Section == "above")])
 })
 
 test_that("con_limit_deviations with constant data", {
@@ -332,30 +373,52 @@ test_that("con_limit_deviations with constant data", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   sd0 <- study_data
   sd0[[prep_map_labels("CRP_0", meta_data, VAR_NAMES, LABEL)]] <- 1.1
-  # with 1, even for DATA_TYPE == float variables, barplots are shown
-  # However, we want  to test a special edge case for histograms
 
   md0 <- meta_data
 
-  MyValueLimits  <- con_limit_deviations(resp_vars  = "CRP_0",
+  expect_silent(
+    MyValueLimits  <- con_limit_deviations(resp_vars  = "CRP_0",
                                          label_col  = "LABEL",
                                          study_data = sd0,
                                          meta_data  = md0,
-                                         limits     = "HARD_LIMITS")
+                                         limits     = "HARD_LIMITS"))
 
   expect_type(MyValueLimits, "list")
 
   skip_on_cran()
   skip_if_not_installed("vdiffr")
-  skip_if_not(capabilities()["long.double"])
+  # TODO: skip_if_not(capabilities()["long.double"])
   suppressWarnings(
     vdiffr::expect_doppelganger(
       "con_limit_deviations 4 hists 1 val",
       MyValueLimits$SummaryPlotList$CRP_0)
   )
+
+  md0[which(md0$LABEL == "CRP_0"), HARD_LIMITS] <- NA
+  md0[which(md0$LABEL == "CRP_0"), DETECTION_LIMITS] <- NA
+  md0[which(md0$LABEL == "CRP_0"), SOFT_LIMITS] <- NA
+  md0[["NEW_LIMITS_ONE"]] <- NA
+  md0[which(md0$LABEL == "CRP_0"), "NEW_LIMITS_ONE"] <- "(-Inf; 1.1)"
+  md0[["NEW_LIMITS_TWO"]] <- NA
+  md0[which(md0$LABEL == "CRP_0"), "NEW_LIMITS_TWO"] <- "(1.1; Inf)"
+
+  expect_silent(
+    MyValueLimits  <- con_limit_deviations(resp_vars  = "CRP_0",
+                                           label_col  = "LABEL",
+                                           study_data = sd0,
+                                           meta_data  = md0,
+                                           limits     = c("NEW_LIMITS_ONE", "NEW_LIMITS_TWO")))
 })
 
 test_that("con_limit_deviations does not crash with missing codes", {
@@ -365,6 +428,14 @@ test_that("con_limit_deviations does not crash with missing codes", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   sd0 <- study_data
   x <- sd0[[prep_map_labels("SBP_0", meta_data, VAR_NAMES, LABEL)]]
@@ -375,56 +446,48 @@ test_that("con_limit_deviations does not crash with missing codes", {
   sd0[[prep_map_labels("SBP_0", meta_data, VAR_NAMES, LABEL)]] <- x
   md0 <- meta_data
 
-  expect_message(
+  expect_silent(
     MyValueLimits  <- con_limit_deviations(resp_vars  = "SBP_0",
                                           label_col  = "LABEL",
                                           study_data = sd0,
                                           meta_data  = md0,
-                                          limits     = "HARD_LIMITS"),
-    regexp = sprintf("(%s|%s)",
-                     paste("The number of bins in the histogram were reduced",
-                           "below \\d+ bins."),
-                     paste("N = 1 values in SBP_0 lie outside hard limits",
-                           "and were removed.")
-                     ),
-    perl = TRUE,
-    all = TRUE
+                                          limits     = "HARD_LIMITS")
   )
 
   expect_type(MyValueLimits, "list")
-  expect_equal(MyValueLimits$SummaryData$`Above hard limits (N)`, 1)
-  expect_equal(MyValueLimits$SummaryData$`Below hard limits (N)`, 0)
+  expect_equal(MyValueLimits$SummaryData$Number[
+    which(MyValueLimits$SummaryData$Section == "above" &
+            MyValueLimits$SummaryData$Limits == "HARD_LIMITS")], 1)
+  expect_equal(MyValueLimits$SummaryData$Number[
+    which(MyValueLimits$SummaryData$Section == "below" &
+            MyValueLimits$SummaryData$Limits == "HARD_LIMITS")], 0)
 
   sd0 <- study_data
   x <- sd0[[prep_map_labels("SBP_0", meta_data, VAR_NAMES, LABEL)]]
 
-  x[[5]] <- 100000 # add a less obvious"forgotten missing code" to the data,
+  x[[5]] <- 100000 # add a less obvious "forgotten missing code" to the data,
   # which causes too many breaks, that should be removed to avoid a crash
 
   sd0[[prep_map_labels("SBP_0", meta_data, VAR_NAMES, LABEL)]] <- x
 
-  expect_message(
+  expect_silent(
     MyValueLimits  <- con_limit_deviations(resp_vars  = "SBP_0",
                                            label_col  = "LABEL",
                                            study_data = sd0,
                                            meta_data  = md0,
-                                           limits     = "HARD_LIMITS"),
-    regexp = sprintf("(%s|%s)",
-                     paste("The number of bins in the histogram were reduced",
-                           "below \\d+ bins."),
-                     paste("N = 1 values in SBP_0 lie outside hard limits",
-                           "and were removed.")
-    ),
-    perl = TRUE,
-    all = TRUE
+                                           limits     = "HARD_LIMITS")
   )
 
   expect_type(MyValueLimits, "list")
-  expect_equal(MyValueLimits$SummaryData$`Above hard limits (N)`, 1)
-  expect_equal(MyValueLimits$SummaryData$`Below hard limits (N)`, 0)
+  expect_equal(MyValueLimits$SummaryData$Number[
+    which(MyValueLimits$SummaryData$Section == "above" &
+            MyValueLimits$SummaryData$Limits == "HARD_LIMITS")], 1)
+  expect_equal(MyValueLimits$SummaryData$Number[
+    which(MyValueLimits$SummaryData$Section == "below" &
+            MyValueLimits$SummaryData$Limits == "HARD_LIMITS")], 0)
 
   skip_on_cran()
-  skip_if_not(capabilities()["long.double"])
+  # TODO: skip_if_not(capabilities()["long.double"])
   skip_if_not_installed("vdiffr")
   suppressWarnings(
     vdiffr::expect_doppelganger(
@@ -441,6 +504,14 @@ test_that("con_limit_deviations does not crash with strong outliers", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   md0 <- meta_data
   md0[["HARD_LIMITS"]][which(md0$LABEL == "BSG_0")] <- "[0;5000]"
@@ -448,26 +519,16 @@ test_that("con_limit_deviations does not crash with strong outliers", {
   x <- sd0[[prep_map_labels("BSG_0", meta_data, VAR_NAMES, LABEL)]]
   x[c(2,8,92)] <- c(26027, 28097, 4031012019)
   sd0[[prep_map_labels("BSG_0", meta_data, VAR_NAMES, LABEL)]] <- x
-  expect_message(
+  expect_silent(
     MyValueLimits  <- con_limit_deviations(resp_vars  = "BSG_0",
                                          label_col  = "LABEL",
                                          study_data = sd0,
                                          meta_data  = md0,
-                                         limits     = "HARD_LIMITS"),
-
-    regexp = sprintf("(%s|%s)",
-                     paste("The number of bins in the histogram were reduced",
-                           "below \\d+ bins."),
-                     paste("N = 3 values in BSG_0 lie outside hard limits and",
-                           "were removed.")
-    ),
-    perl = TRUE,
-    all = TRUE
+                                         limits     = "HARD_LIMITS")
   )
 
   expect_type(MyValueLimits, "list")
-  expect_equal(MyValueLimits$SummaryData$`Above hard limits (N)`, 3)
-  expect_equal(MyValueLimits$SummaryData$`Below hard limits (N)`, 0)
+  expect_equal(MyValueLimits$SummaryTable$NUM_con_rvv_inum, 3)
 })
 
 test_that("con_limits_deviations does not crash with NAs in datetime vars", {
@@ -478,25 +539,30 @@ test_that("con_limits_deviations does not crash with NAs in datetime vars", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   #sd0 <- study_data
   #x <- sd0[[prep_map_labels("QUEST_DT_0", meta_data, VAR_NAMES, LABEL)]]
   #any(is.na(x))
   #length(which(is.na(x)))
   # Variable QUEST_DT_0 already contains NA values
-  expect_message(
+  expect_silent(
     MyValueLimits  <- con_limit_deviations(resp_vars  = "QUEST_DT_0",
                                          label_col  = "LABEL",
                                          study_data = study_data,
                                          meta_data  = meta_data,
-                                         limits     = "HARD_LIMITS"),
-    regexp = paste("N = 9 values in QUEST_DT_0 lie outside hard limits",
-                   "and were removed.")
+                                         limits     = "HARD_LIMITS")
   )
 
   expect_type(MyValueLimits, "list")
-  expect_equal(MyValueLimits$SummaryData$`Above hard limits (N)`, 0)
-  expect_equal(MyValueLimits$SummaryData$`Below hard limits (N)`, 9)
+  expect_equal(MyValueLimits$SummaryTable$NUM_con_rvv_itdat, 9)
 })
 
 test_that("con_limit_deviations works with no values within limits", {
@@ -506,31 +572,31 @@ test_that("con_limit_deviations works with no values within limits", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   md0 <- meta_data
   md0[["HARD_LIMITS"]][which(md0$LABEL == "BSG_0")] <- "[51.5;51.8]"
   sd0 <- study_data
 
-  expect_message(
+  expect_silent(
     MyValueLimits  <- con_limit_deviations(resp_vars  = "BSG_0",
                                          label_col  = "LABEL",
                                          study_data = sd0,
                                          meta_data  = md0,
-                                         limits     = "HARD_LIMITS"),
-    regexp = sprintf("(%s|%s)",
-                     paste("The number of bins in the histogram were reduced",
-                           "below \\d+ bins."),
-                     paste("N = 2686 values in BSG_0 lie outside hard limits and",
-                           "were removed.")
-    ),
-    perl = TRUE,
-    all = TRUE
+                                         limits     = "HARD_LIMITS")
   )
 
   expect_type(MyValueLimits, "list")
   skip_on_cran()
   skip_if_not_installed("vdiffr")
-  skip_if_not(capabilities()["long.double"])
+  # TODO: skip_if_not(capabilities()["long.double"])
   suppressWarnings(
     vdiffr::expect_doppelganger(
       "con_limit_deviations_nothing_within",
@@ -545,17 +611,26 @@ test_that("con_limit_deviations works with wrong datetime limits", {
 
   meta_data <- prep_get_data_frame("meta_data")
   study_data <- prep_get_data_frame("study_data")
+  meta_data2 <-
+    prep_scalelevel_from_data_and_metadata(study_data = study_data,
+                                           meta_data = meta_data)
+  meta_data[[SCALE_LEVEL]] <-
+    setNames(meta_data2[[SCALE_LEVEL]], nm = meta_data2[[VAR_NAMES]])[
+      meta_data[[VAR_NAMES]]
+    ]
+
 
   md0 <- meta_data
   md0[["HARD_LIMITS"]][which(md0$LABEL == "QUEST_DT_0")] <- "[51.5;51.8]"
   sd0 <- study_data
 
-  expect_message(
-    MyValueLimits <- con_limit_deviations(resp_vars  = "QUEST_DT_0",
-                                          label_col  = "LABEL",
-                                          study_data = sd0,
-                                          meta_data  = md0,
-                                          limits     = "HARD_LIMITS"))
+  expect_warning(
+      MyValueLimits <- con_limit_deviations(resp_vars  = "QUEST_DT_0",
+                                            label_col  = "LABEL",
+                                            study_data = sd0,
+                                            meta_data  = md0,
+                                            limits     = "HARD_LIMITS"),
+  regexp = "Invalid limits detected")
 
   md0[["HARD_LIMITS"]][which(md0$LABEL == "QUEST_DT_0")] <-
     "[2018-10-01; 2017-10-01]"
