@@ -18,6 +18,9 @@ test_that("prep_study2meta works", {
   reference <- data.frame(VAR_NAMES = colnames(cars),
                           LABEL = colnames(cars),
                           DATA_TYPE = "integer",
+                          SCALE_LEVEL = c(SCALE_LEVELS$ORDINAL, # this is not correctly guessed
+                                          SCALE_LEVELS$RATIO),
+                          UNIT = NA_character_,
                           VALUE_LABELS = NA_character_,
                           MISSING_LIST_TABLE = NA_character_,
                           HARD_LIMITS = NA_character_,
@@ -58,6 +61,9 @@ test_that("prep_study2meta handles tibbles correctly", {
                data.frame(VAR_NAMES = colnames(cars),
                           LABEL = colnames(cars),
                           DATA_TYPE = "integer",
+                          SCALE_LEVEL = c(SCALE_LEVELS$ORDINAL, # this is not correctly guessed
+                                          SCALE_LEVELS$RATIO),
+                          UNIT = NA_character_,
                           VALUE_LABELS = NA_character_,
                           MISSING_LIST_TABLE = NA_character_,
                           HARD_LIMITS = NA_character_,
@@ -94,6 +100,9 @@ test_that("prep_study2meta handles tibbles correctly", {
                data.frame(VAR_NAMES = colnames(cars),
                           LABEL = colnames(cars),
                           DATA_TYPE = "integer",
+                          SCALE_LEVEL = c(SCALE_LEVELS$ORDINAL, # this is not correctly guessed
+                                          SCALE_LEVELS$RATIO),
+                          UNIT = NA_character_,
                           VALUE_LABELS = NA_character_,
                           MISSING_LIST_TABLE = NA_character_,
                           HARD_LIMITS = NA_character_,
@@ -130,6 +139,62 @@ test_that("prep_study2meta works on study_data", {
       VAR_NAMES = meta_data$VAR_NAMES,
       LABEL = meta_data$VAR_NAMES,
       DATA_TYPE = meta_data$DATA_TYPE,
+      SCALE_LEVEL = c(
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$`NA`,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$RATIO,
+        SCALE_LEVELS$RATIO,
+        SCALE_LEVELS$RATIO,
+        SCALE_LEVELS$RATIO,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$RATIO,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$INTERVAL,
+        SCALE_LEVELS$RATIO,
+        SCALE_LEVELS$RATIO,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$INTERVAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$INTERVAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$INTERVAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$RATIO,
+        SCALE_LEVELS$ORDINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$NOMINAL,
+        SCALE_LEVELS$NOMINAL
+      ),
+      UNIT = NA_character_,
       VALUE_LABELS = NA_character_,
       MISSING_LIST_TABLE = NA_character_,
       HARD_LIMITS = NA_character_,
