@@ -1,13 +1,12 @@
-test_that("multiplication works", {
-     load(system.file("extdata/meta_data.RData", package = "dataquieR"),
-       envir = environment())
-     load(system.file("extdata/study_data.RData", package = "dataquieR"),
-       envir = environment())
-     meta_data[meta_data[[LABEL]] == "EATING_PREFS_0", RECODE] <-
+test_that("util_dichotomize works", {
+  skip_on_cran() # not used, yet
+  meta_data <- prep_get_data_frame("meta_data")
+  study_data <- prep_get_data_frame("study_data")
+  meta_data[meta_data[[LABEL]] == "EATING_PREFS_0", RECODE] <-
        "0" # 0 = "eat meat" as reference
      meta_data[meta_data[[LABEL]] == "MEAT_CONS_0", RECODE] <-
        "1| 2|3 | 4 " # eats meat more than 0="never"
-     m_study_data <- util_replace_codes_by_NA(study_data, meta_data)$study_data
+     m_study_data <- util_replace_codes_by_NA(study_data, meta_data)
      d_study_data <- util_dichotomize(m_study_data, meta_data)
      x <- d_study_data[,
        prep_map_labels(c("EATING_PREFS_0", "MEAT_CONS_0"),
